/*
 * File         : spmd_Remap.C
 * Author       : Guansong Zhang
 * Created      : Mon Oct 27 16:31:17 1997
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/01/30 17:53:33 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#include "spmd_Remap.h"
#include "adlib.h"

DAD * spmd_DAD_ResolvePtr(JNIEnv *, jobject);

extern jfieldID hCPPObjID;
#include "jctable.h"
jctable<Remap> spmd_Remap_Table;

/*ResolvePtr
 */
Remap * spmd_Remap_ResolvePtr(JNIEnv *env, jobject jthis) {
  return(spmd_Remap_Table.ObjPtr[env->GetIntField(jthis, hCPPObjID)]);
}

/*
 * Class:     spmd_Remap
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Remap_init
  (JNIEnv * env, jclass jthis)
{
  jclass InterfaceObject_class=env->FindClass("spmd/InterfaceObject");
  hCPPObjID=env->GetFieldID(InterfaceObject_class, "hCPPObj", "I");  
  jctable<Remap> *tempspmd_Remap_Table=new jctable<Remap>;
  spmd_Remap_Table=*tempspmd_Remap_Table;
  delete tempspmd_Remap_Table;
};

/*
 * Class:     spmd_Remap
 * Method:    clean
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Remap_clean
  (JNIEnv *, jclass);

/*
 * Class:     spmd_Remap
 * Method:    constructor
 * Signature: (Lspmd/DAD;Lspmd/DAD;I)V
 */
JNIEXPORT void JNICALL Java_spmd_Remap_constructor
  (JNIEnv *env, jobject jthis, jobject dDAD, jobject sDAD, jint len)
{
  DAD * dDADPtr = spmd_DAD_ResolvePtr(env, dDAD); 
  DAD * sDADPtr = spmd_DAD_ResolvePtr(env, sDAD); 

  Remap *cppRemap=new Remap(dDADPtr, sDADPtr, len);
  long pos= spmd_Remap_Table.NewHandle(cppRemap);

  if (spmd_Remap_Table.ObjPtr[pos]==NULL) {
    jclass cError=env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError,"adlib memory allocation failed");
    return;
  }

  //spmd_Remap_IncRefCntr(pos);
  
  env->SetIntField(jthis, hCPPObjID, pos);  

};

/*
 * Class:     spmd_Remap
 * Method:    finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Remap_finalize
  (JNIEnv * env, jobject jthis) 
{
  long thecppobj=env->GetIntField(jthis, hCPPObjID);
  long rc=spmd_Remap_Table.DecRefCntr(thecppobj);
  if (rc==0) {
    delete spmd_Remap_Table.ObjPtr[thecppobj];
    spmd_Remap_Table.ObjPtr[thecppobj]=NULL;
  }
};

/*
 * Class:     spmd_Remap
 * Method:    execute
 * Signature: ([IJ[IJ)V
 */
JNIEXPORT void JNICALL Java_spmd_Remap_execute___3IJ_3IJ
  (JNIEnv * env, jobject jthis, jintArray dst, jlong dOffset, jintArray src, jlong sOffset)
{
  jboolean isCopy;

  int dLength=env->GetArrayLength(dst);
  jint * jDA=env->GetIntArrayElements(dst, &isCopy);
    
  int sLength=env->GetArrayLength(src);
  jint * jSA=env->GetIntArrayElements(src, &isCopy);

  /*
  {//debug
    int i;

    for (i=0; i<dLength; i++) {
      printf("dst %d: %d | ", i, *(jDA+i));
    }
    printf("\n");
    
    for (i=0; i<sLength; i++) {
      printf("src %d: %d | ", i, *(jSA+i));
    }
    printf("\n");
  }
  */

  spmd_Remap_ResolvePtr(env, jthis)->execute(jDA+dOffset, jSA+sOffset); 

  //printf("recorded remap at %x; dOffset %d; sOffset %d\n", spmd_Remap_ResolvePtr(env, jthis), dOffset, sOffset);  //debug

  /*
  {//debug
    int i;

    for (i=0; i<dLength; i++) {
      printf("dst- %d: %d | ", i, *(jDA+i));
    }
    printf("\n");
    
    for (i=0; i<sLength; i++) {
      printf("src- %d: %d | ", i, *(jSA+i));
    }
    printf("\n");
  }
  */

  env->ReleaseIntArrayElements(dst, jDA, 0);
  env->ReleaseIntArrayElements(src, jSA, JNI_ABORT);
};

/*
 * Class:     spmd_Remap
 * Method:    execute
 * Signature: ([FJ[FJ)V
 */
JNIEXPORT void JNICALL Java_spmd_Remap_execute___3FJ_3FJ
  (JNIEnv * env, jobject jthis, jfloatArray dst, jlong dOffset, jfloatArray src, jlong sOffset)
{
  jboolean isCopy;

  int dLength=env->GetArrayLength(dst);
  jfloat * jDA=env->GetFloatArrayElements(dst, &isCopy);
    
  int sLength=env->GetArrayLength(src);
  jfloat * jSA=env->GetFloatArrayElements(src, &isCopy);

  /*  {//debug
    int i;

    AdlibPrintf("dst ");
    for (i=0; i<dLength; i++) {
      printf("%d: %f | ", i, *(jDA+i));
    }
    printf("\n");

    AdlibPrintf("src ");
    for (i=0; i<sLength; i++) {
      printf("%d: %f | ", i, *(jSA+i));
    }
    printf("\n");
  }
  
  */
  spmd_Remap_ResolvePtr(env, jthis)->execute(jDA+dOffset, jSA+sOffset); 

  /*{//debug
    int i;

    AdlibPrintf("dst- ");
    for (i=0; i<dLength; i++) {
      printf(" %d: %f | ", i, *(jDA+i));
    }
    printf("\n");

    AdlibPrintf("src- ");
    for (i=0; i<sLength; i++) {
      printf("%d: %f | ", i, *(jSA+i));
    }
    printf("\n");
  }
  */

  env->ReleaseFloatArrayElements(dst, jDA, 0);
  env->ReleaseFloatArrayElements(src, jSA, JNI_ABORT);
};
